<?php
namespace Chatwee\Chatwee\Hooks;

use Chatwee\Chatwee\Sso\SsoConfiguration;
use Chatwee\Chatwee\Sso\SsoSession;
use Chatwee\Chatwee\Sso\SsoUser;
use Chatwee\Chatwee\Sso\SsoManager;

class ChatweeListener
{
    public function renderPreProcess($params, $pObj)
    {
        $settings = unserialize($GLOBALS['TYPO3_CONF_VARS']['EXT']['extConf']['chatwee']);

        if($settings['chatweeCode'])
    	{
    		$pObj->addHeaderData($settings['chatweeCode']);
    	}
    }

    public function contentPostProcOutput($params, $pObj)
    {
        if (!$GLOBALS['TSFE']->isINTincScript()) {
            return;
        }

        $settings = unserialize($GLOBALS['TYPO3_CONF_VARS']['EXT']['extConf']['chatwee']);

    	if($settings['enableSso'])
    	{
    		$this->synchronizeSessions();
    	}
    }

    public function contentPostProcAll($params, $pObj)
    {
        if ($GLOBALS['TSFE']->isINTincScript()) {
            return;
        }

        $settings = unserialize($GLOBALS['TYPO3_CONF_VARS']['EXT']['extConf']['chatwee']);

    	if($settings['enableSso'])
    	{
    		$this->synchronizeSessions();
    	}
    }

    public function logOffPreProcessing(array $params, $auth_obj)
    {
    	$settings = unserialize($GLOBALS['TYPO3_CONF_VARS']['EXT']['extConf']['chatwee']);

    	if($settings['enableSso'])
    	{
    		$this->setSsoCredentials();
			$this->ssoLogout();
		}
    }

    public function postUserLookUp($params, $auth_obj)
    {
        if ($auth_obj->loginSessionStarted)
        {
        	$settings = unserialize($GLOBALS['TYPO3_CONF_VARS']['EXT']['extConf']['chatwee']);

	    	if($settings['enableSso'])
	    	{
        		$this->setSsoCredentials();
				$this->ssoLogin();
			}
        }
    }

	public function setSsoCredentials()
	{
		try
		{
			$settings = unserialize($GLOBALS['TYPO3_CONF_VARS']['EXT']['extConf']['chatwee']);

			SsoConfiguration::setChatId($settings["chatId"]);
			SsoConfiguration::setClientKey($settings["clientKey"]);
			SsoConfiguration::setCustomUserAgent("Chatwee Typo3 Plugin 0.9");
		}
		catch(Exception $exception)
		{
			echo "An error occured: " . $exception->getMessage();
		}
	}

	public function ssoLogin()
	{
		$user = $GLOBALS['TSFE']->fe_user->user;

		$userId = $user['uid'];

		$chatweeUserId = self::getChatweeUserId($userId);

		if(!$chatweeUserId)
		{
			$chatweeUserId = self::ssoRegister($userId);
		}
		if(!$chatweeUserId)
		{
			return false;
		}

		try
		{
			SsoManager::editUser(array(
				'userId' => $chatweeUserId,
				'login'	=> $user["username"],
				'avatar' => "",
				'isAdmin' => false
			));
		}
		catch(Exception $exception)
		{
			echo "An error occured: " . $exception->getMessage();
			return false;
		}

		try
		{
			$sessionId = SsoManager::loginUser(array(
				'userId' => $chatweeUserId
			));
		}
		catch(Exception $exception)
		{
			echo "An error occured: " . $exception->getMessage();
			return false;
		}
	}

	public function getChatweeUserId($userId)
	{
		try
		{
			$dbres = $GLOBALS['TYPO3_DB']->exec_SELECTquery('*', 'tx_chatwee_users', "uid = '" . $userId . "'");

			if (!$dbres) {
				return false;
			}
	        $user = $GLOBALS['TYPO3_DB']->sql_fetch_assoc($dbres);

	        if(!$user) {
	        	return null;
	        }

	        return $user["chatwee_uid"];
		}
		catch(Exception $exception)
		{
			echo "An error occured: " . $exception->getMessage();
			return false;
		}
	}

	public function ssoRegister($userId)
	{
		$user = $GLOBALS['TSFE']->fe_user->user;

		try
		{
			$chatweeUserId = SsoManager::registerUser(array(
				'login' => $user["username"],
				'avatar' => "",
				'isAdmin' => false
			));

			self::assignChatweeUserId($userId, $chatweeUserId);
		}
		catch(Exception $exception)
		{
			echo "An error occured: " . $exception->getMessage();
			return false;
		}
		return $chatweeUserId;
	}

	public function assignChatweeUserId($userId, $chatweeUserId)
	{
		try
		{
	        $properties = array(
	            'uid' => $userId,
	            'chatwee_uid' => $chatweeUserId
	        );

	        $GLOBALS['TYPO3_DB']->exec_INSERTquery('tx_chatwee_users', $properties);
	        $uid = $GLOBALS['TYPO3_DB']->sql_insert_id();
		}
		catch(Exception $exception)
		{
			echo "An error occured: " . $exception->getMessage();
			return false;
		}
	}

	public function ssoLogout()
	{
		try
		{
			$sessionId = SsoManager::logoutUser();
		}
		catch(Exception $exception)
		{
			$error = "An error occured: " . $exception->getMessage();
			return false;
		}
	}

	public function synchronizeSessions()
	{
		$this->setSsoCredentials();

		$isUserLoggedIn = $GLOBALS['TSFE']->fe_user->user ? true : false;

		if($isUserLoggedIn === true && SsoSession::isSessionSet() === false)
		{
			self::ssoLogin();
		}
		if($isUserLoggedIn === true && SsoSession::isSessionSet() === true)
		{
			try
			{
				$isSessionValid = SsoUser::validateSession(array(
					'sessionId' => SsoSession::getSessionId(),
				));
			}
			catch(Exception $exception)
			{
				echo "An error occured: " . $exception->getMessage();
				return false;
			}

			if($isSessionValid === false)
			{
				self::ssoLogin();
			}
		}
	}
}
